@@ -1,5 +1,16 @@
 
-# $rcs = ' $Id: Charset.pm,v 1.37 2004/01/31 01:34:47 Daddy Exp $ ' ;
+# $rcs = ' $Id: Charset.pm,v 1.382 2006/12/09 01:38:07 Daddy Exp $ ' ;
+
+package I18N::Charset;
+
+require 5.002;
+
+require Exporter;
+use Carp;
+use IO::String;
+
+# Don't leave home without it:
+use strict;
 
 =head1 NAME
 
@@ -29,16 +40,6 @@ I18N::Charset - IANA Character Set Registry names and Unicode::MapUTF8
   I18N::Charset::add_libi_alias('my-sjis' => 'x-sjis');
   I18N::Charset::add_enco_alias('my-japanese' => 'shiftjis');
 
-=cut
-
-#-----------------------------------------------------------------------
-
-package I18N::Charset;
-
-require 5.002;
-
-#-----------------------------------------------------------------------
-
 =head1 DESCRIPTION
 
 The C<I18N::Charset> module provides access to the IANA Character Set
@@ -63,20 +64,11 @@ functions will always return undef.
 
 =cut
 
-#-----------------------------------------------------------------------
-
-require Exporter;
-use Carp;
-use IO::String;
-
-# Don't leave home without it:
-use strict;
-
 #-----------------------------------------------------------------------
 #	Public Global Variables
 #-----------------------------------------------------------------------
 use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK );
-$VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/o);
+$VERSION = do { my @r = (q$Revision: 1.382 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 @ISA       = qw( Exporter );
 @EXPORT    = qw( iana_charset_name
 map8_charset_name
@@ -94,14 +86,14 @@ mib_to_charset_name charset_name_to_mib
 #	Private Global Variables
 #-----------------------------------------------------------------------
 
-# %SHORTtoMIB is a hash of stripped names to mib.
-my %SHORTtoMIB;
-# %MIBtoLONG is a hash of mib to long name.
-my %MIBtoLONG;
-# %LONGtoMIB is a hash of long name to mib.
-my %LONGtoMIB;
-# %MIBtoMIME is a hash of mib to preferred MIME names.
-my %MIBtoMIME;
+# %hsMIBofShortname is a hash of stripped names to mib.
+my %hsMIBofShortname;
+# %hsLongnameOfMIB is a hash of mib to long name.
+my %hsLongnameOfMIB;
+# %hsMIBofLongname is a hash of long name to mib.
+my %hsMIBofLongname;
+# %hsMIMEofMIB is a hash of mib to preferred MIME names.
+my %hsMIMEofMIB;
 # %MIBtoMAP8 is a hash of mib to Unicode::Map8 names.  (Only valid for
 # those U::Map8 names that we can find in the IANA registry)
 my %MIBtoMAP8;
@@ -116,11 +108,12 @@ my %MIBtoUMU8;
 # %MIBtoLIBI is a hash of mib to libiconv names.  (Only valid for
 # those libiconv names that we can find in the IANA registry)
 my %MIBtoLIBI;
-# %MIBtoLIBI is a hash of mib to Encode names.  (Only valid for
+# %MIBtoENCO is a hash of mib to Encode names.  (Only valid for
 # those Encode names that we can find in the IANA registry)
 my %MIBtoENCO;
 
 use constant DEBUG => 0;
+use constant DEBUG_ENCO => 0;
 
 =head1 CONVERSION ROUTINES
 
@@ -158,34 +151,34 @@ sub iana_charset_name
   return undef unless $code ne '';
   # $iDebug = ($code =~ m!sjis!);
   # print STDERR " + iana_charset_name($code)..." if $iDebug;
-  my $mib = &short_to_mib($code);
+  my $mib = &_short_to_mib($code);
   return undef unless defined $mib;
   # print STDERR " + mib is ($mib)..." if $iDebug;
   # Make sure this is really a IANA mib:
   return undef if &_is_dummy($mib);
   # print STDERR " + is really iana..." if $iDebug;
-  return $MIBtoLONG{$mib};
+  return $hsLongnameOfMIB{$mib};
   } # iana_charset_name
 
 
-sub try_list
+sub _try_list
   {
   my $code = shift;
-  my @asTry = ($code, &strip($code));
-  push @asTry, &strip($code) if $code =~ s!\A(x-)+!!;  # try without leading x-
+  my @asTry = ($code, &_strip($code));
+  push @asTry, &_strip($code) if $code =~ s!\A(x-)+!!;  # try without leading x-
   return @asTry;
-  } # try_list
+  } # _try_list
 
-sub short_to_mib
+sub _short_to_mib
   {
   my $code = shift;
   local $^W = 0;
-  # print STDERR " + short_to_mib($code)..." if DEBUG;
+  # print STDERR " + _short_to_mib($code)..." if DEBUG;
   my $answer = undef;
  TRY_SHORT:
-  foreach my $sTry (&try_list($code))
+  foreach my $sTry (&_try_list($code))
     {
-    my $iMIB = $SHORTtoMIB{$sTry} || 'undef';
+    my $iMIB = $hsMIBofShortname{$sTry} || 'undef';
     # print STDERR "try($sTry)...$iMIB..." if DEBUG;
     if ($iMIB ne 'undef')
       {
@@ -195,16 +188,16 @@ sub short_to_mib
     } # foreach
   # print STDERR "answer is $answer\n" if DEBUG;
   return $answer;
-  } # short_to_mib
+  } # _short_to_mib
 
 
-sub short_to_long
+sub _short_to_long
   {
   local $^W = 0;
   my $s = shift;
-  # print STDERR " + short_to_long($s)..." if DEBUG;
-  return $MIBtoLONG{&short_to_mib($s)};
-  } # short_to_long
+  # print STDERR " + _short_to_long($s)..." if DEBUG;
+  return $hsLongnameOfMIB{&_short_to_mib($s)};
+  } # _short_to_long
 
 
 =item mime_charset_name()
@@ -226,13 +219,13 @@ sub mime_charset_name
   return undef unless defined $code;
   return undef unless $code ne '';
   # print STDERR " + mime_charset_name($code)..." if DEBUG;
-  my $mib = &short_to_mib($code);
+  my $mib = &_short_to_mib($code);
   return undef unless defined $mib;
   # print STDERR " + mib is ($mib)..." if DEBUG;
   # Make sure this is really an IANA mib:
   return undef if &_is_dummy($mib);
   # print STDERR " + is really iana..." if DEBUG;
-  return $MIBtoMIME{$mib};
+  return $hsMIMEofMIB{$mib};
   } # mime_charset_name
 
 
@@ -257,7 +250,11 @@ sub _maybe_load_enco # PRIVATE
   # Get a list of aliases from Encode:
   if (eval q{require Encode})
     {
-    my @as = Encode->encodings(':all');
+    my @as;
+    @as = Encode->encodings(':all');
+    # push @as, Encode->encodings('EBCDIC');
+    my $iFake = 0;
+    my $iReal = 0;
  ENCODING:
     foreach my $s (@as)
       {
@@ -268,18 +265,28 @@ sub _maybe_load_enco # PRIVATE
         {
         # Create a dummy mib:
         $mib = $sFakeMIB++;
+        $iFake++;
         } # if
       else
         {
         $mib = &charset_name_to_mib($sIana);
+        $iReal++;
         }
       # At this point we have a mib for this Encode entry.
       $MIBtoENCO{$mib} = $s;
-      # print STDERR " +   mib for enco ==$s== is $mib\n";
-      $SHORTtoMIB{&strip($s)} = $mib;
-      # print STDERR " +   assign enco =$s==>$mib\n" if &_is_dummy($mib);
+      DEBUG_ENCO && print STDERR " +   mib for enco ==$s== is $mib\n";
+      $hsMIBofShortname{&_strip($s)} = $mib;
+      DEBUG_ENCO && print STDERR " +   assign enco =$s==>$mib\n" if &_is_dummy($mib);
+      } # foreach ENCODING
+    if (DEBUG_ENCO)
+      {
+      print STDERR " + Summary of Encode encodings:\n";
+      printf STDERR (" +   %d encodings found.\n", scalar(@as));
+      print STDERR " +   $iFake fake mibs created.\n";
+      print STDERR " +   $iReal real mibs re-used.\n";
       } # if
     $iEncoLoaded = 1;
+    &add_enco_alias('Windows-31J', 'cp932');
     } # if
   else
     {
@@ -300,12 +307,12 @@ sub enco_charset_name
   return undef unless $code ne '';
   &_maybe_load_enco();
   my $iDebug = 0; # ($code =~ m!johab!i);
-  print STDERR " + enco_charset_name($code)..." if $iDebug;
-  my $mib = &short_to_mib($code);
+  print STDERR " + enco_charset_name($code)..." if ($iDebug || DEBUG_ENCO);
+  my $mib = &_short_to_mib($code);
   return undef unless defined $mib;
-  print STDERR " + mib is ($mib)..." if $iDebug;
+  print STDERR " + mib is ($mib)..." if ($iDebug || DEBUG_ENCO);
   my $ret = &_mib_to_enco($mib);
-  print STDERR " + enco is ($ret)..." if $iDebug;
+  print STDERR " + enco is ($ret)..." if ($iDebug || DEBUG_ENCO);
   return $ret;
   } # enco_charset_name
 
@@ -366,7 +373,7 @@ sub _maybe_load_libi # PRIVATE
           {
           $MIBtoLIBI{$mib} = $sWord;
           # print STDERR " +   mib for libi ==$sWord== is $mib\n";
-          $SHORTtoMIB{&strip($sWord)} = $mib;
+          $hsMIBofShortname{&_strip($sWord)} = $mib;
           } # foreach ADD_LIBI
         } # foreach ICONV_LINE
       } # if
@@ -387,7 +394,7 @@ sub libi_charset_name
   return undef unless $code ne '';
   # my $iDebug = 1; # ($code =~ m!johab!i);
   # print STDERR " + libi_charset_name($code)..." if $iDebug;
-  my $mib = &short_to_mib($code);
+  my $mib = &_short_to_mib($code);
   return undef unless defined $mib;
   # print STDERR " + mib is ($mib)..." if $iDebug;
   my $ret = &_mib_to_libi($mib);
@@ -418,7 +425,7 @@ sub mib_to_charset_name
   return undef unless defined $code;
   return undef unless $code ne '';
   local $^W = 0;
-  return $MIBtoLONG{$code};
+  return $hsLongnameOfMIB{$code};
   } # mib_to_charset_name
 
 
@@ -439,7 +446,7 @@ sub charset_name_to_mib
   {
   my $s = shift;
   return undef unless defined($s);
-  return $LONGtoMIB{$s} || $LONGtoMIB{
+  return $hsMIBofLongname{$s} || $hsMIBofLongname{
                                       &iana_charset_name($s) ||
                                       &umap_charset_name($s) ||
                                       &map8_charset_name($s) ||
@@ -468,9 +475,9 @@ sub map8_charset_name
   return undef unless $code ne '';
   # $iDebug = 0 && ($code =~ m!037!);
   # print STDERR " + map8_charset_name($code)..." if $iDebug;
-  $code = &strip($code);
+  $code = &_strip($code);
   # print STDERR "$code..." if $iDebug;
-  my $iMIB = &short_to_mib($code) || 'undef';
+  my $iMIB = &_short_to_mib($code) || 'undef';
   # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
@@ -492,8 +499,6 @@ important.
 
     $sCharset = umap_charset_name('hebrew');
 
-=back
-
 =cut
 
 sub umap_charset_name
@@ -504,7 +509,7 @@ sub umap_charset_name
   # $iDebug = ($code =~ m!apple!i);
   # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{$sDummy .'029'}\n\n" if $iDebug;
   # print STDERR " + umap_charset_name($code)..." if $iDebug;
-  my $iMIB = &short_to_mib(&strip($code)) || 'undef';
+  my $iMIB = &_short_to_mib(&_strip($code)) || 'undef';
   # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
@@ -536,7 +541,7 @@ sub umu8_charset_name
   return undef unless $code ne '';
   # $iDebug = ($code =~ m!u!);
   # print STDERR " + umu8_charset_name($code)..." if $iDebug;
-  my $iMIB = &short_to_mib($code) || 'undef';
+  my $iMIB = &_short_to_mib($code) || 'undef';
   # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
@@ -547,6 +552,7 @@ sub umu8_charset_name
   return undef;
   } # umu8_charset_name
 
+=back
 
 =head1 QUERY ROUTINES
 
@@ -566,7 +572,7 @@ The names are not in any particular order.
 
 sub all_iana_charset_names
   {
-  return values %MIBtoLONG;
+  return values %hsLongnameOfMIB;
   } # all_iana_charset_names
 
 #-----------------------------------------------------------------------
@@ -609,12 +615,12 @@ sub add_iana_alias
   my $sName = &iana_charset_name($sReal);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown IANA character set \"$sReal\"\n" if DEBUG;
+    carp qq{attempt to alias "$sAlias" to unknown IANA charset "$sReal"};
     return undef;
     } # if
-  my $mib = &short_to_mib(&strip($sName));
+  my $mib = &_short_to_mib(&_strip($sName));
   # print STDERR " --> $sName --> $mib\n";
-  $SHORTtoMIB{&strip($sAlias)} = $mib;
+  $hsMIBofShortname{&_strip($sAlias)} = $mib;
   return $sName;
   } # add_iana_alias
 
@@ -660,16 +666,16 @@ sub add_map8_alias
   {
   my ($sAlias, $sReal) = @_;
   my $sName = &map8_charset_name($sReal);
-  my $sShort = &strip($sAlias);
-  my $sShortName = &strip($sName);
+  my $sShort = &_strip($sAlias);
+  my $sShortName = &_strip($sName);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown Map8 character set \"$sReal\"\n" if DEBUG;
+    carp qq{attempt to alias "$sAlias" to unknown Map8 charset "$sReal"};
     return undef;
     } # if
-  if (exists $SHORTtoMIB{$sShortName})
+  if (exists $hsMIBofShortname{$sShortName})
     {
-    $SHORTtoMIB{$sShort} = $SHORTtoMIB{$sShortName};
+    $hsMIBofShortname{$sShort} = $hsMIBofShortname{$sShortName};
     } # if
   return $sName;
   } # add_map8_alias
@@ -687,16 +693,16 @@ sub add_umap_alias
   {
   my ($sAlias, $sReal) = @_;
   my $sName = &umap_charset_name($sReal);
-  my $sShort = &strip($sAlias);
-  my $sShortName = &strip($sName);
+  my $sShort = &_strip($sAlias);
+  my $sShortName = &_strip($sName);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown U::Map character set \"$sReal\"\n" if DEBUG;
+    carp qq{attempt to alias "$sAlias" to unknown U::Map charset "$sReal"};
     return undef;
     } # if
-  if (exists $SHORTtoMIB{$sShortName})
+  if (exists $hsMIBofShortname{$sShortName})
     {
-    $SHORTtoMIB{$sShort} = $SHORTtoMIB{$sShortName};
+    $hsMIBofShortname{$sShort} = $hsMIBofShortname{$sShortName};
     } # if
   return $sName;
   } # add_umap_alias
@@ -735,12 +741,12 @@ sub add_libi_alias
   my $sName = &libi_charset_name($sReal);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown iconv conversion scheme \"$sReal\"\n" if DEBUG;
+    carp qq{attempt to alias "$sAlias" to unknown iconv charset "$sReal"};
     return undef;
     } # if
-  my $mib = &short_to_mib(&strip($sName));
+  my $mib = &_short_to_mib(&_strip($sName));
   # print STDERR "sName=$sName...mib=$mib\n";
-  $SHORTtoMIB{&strip($sAlias)} = $mib;
+  $hsMIBofShortname{&_strip($sAlias)} = $mib;
   return $sName;
   } # add_libi_alias
 
@@ -775,22 +781,22 @@ sub add_enco_alias
   {
   my ($sAlias, $sReal) = @_;
   my $iDebug = 0;
-  print STDERR " + add_enco_alias($sAlias,$sReal)..." if $iDebug;
+  print STDERR " + add_enco_alias($sAlias,$sReal)..." if ($iDebug || DEBUG_ENCO);
   my $sName = &enco_charset_name($sReal);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown iconv conversion scheme \"$sReal\"\n" if DEBUG;
+    carp qq{attempt to alias "$sAlias" to unknown Encode charset "$sReal"};
     return undef;
     } # if
-  my $mib = &short_to_mib(&strip($sName));
-  print STDERR "sName=$sName...mib=$mib\n" if $iDebug;
-  $SHORTtoMIB{&strip($sAlias)} = $mib;
+  my $mib = &_short_to_mib(&_strip($sName));
+  print STDERR "sName=$sName...mib=$mib\n" if ($iDebug || DEBUG_ENCO);
+  $hsMIBofShortname{&_strip($sAlias)} = $mib;
   return $sName;
   } # add_enco_alias
 
 #-----------------------------------------------------------------------
 
-=head1 EXAMPLES
+=back
 
 =head1 KNOWN BUGS AND LIMITATIONS
 
@@ -844,14 +850,13 @@ A wrapper around all three of these character set conversion distributions.
 
 =back
 
-
 =head1 AUTHOR
 
-Martin Thurn E<lt>mthurn@cpan.orgE<gt>
+Martin Thurn, C<mthurn@cpan.org>, L<http://tinyurl.com/nn67z>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 1998-2001 TASC, Inc.
+Copyright (c) 1998-2006 Martin Thurn
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -860,12 +865,12 @@ modify it under the same terms as Perl itself.
 
 #-----------------------------------------------------------------------
 
-sub strip
+sub _strip
   {
   my $s = lc(shift);
   $s =~ tr/[0-9a-zA-Z]//dc;
   return $s;
-  } # strip
+  } # _strip
 
 # initialization code - stuff the DATA into some data structure
 
@@ -877,7 +882,9 @@ while (1)
   my ($sName, $iMIB, $sAlias, $mimename);
   my $iDebug = 0;
   # I used to use the __DATA__ mechanism to initialize the data, but
-  # that is not compatible with perlapp
+  # that is not compatible with perlapp.  NOTE that storing the IANA
+  # charset data as a file separate from this module code will not
+  # work with perlapp either!
   my $io = new IO::String(_init_data());
   local $/ = "\n";
  DATA:
@@ -906,13 +913,13 @@ while (1)
       $iMIB = $1;
       # $iDebug = ($iMIB =~ m!225[23]!);
       print STDERR " +   found mib: $iMIB, long = $sName\n" if $iDebug;
-      $MIBtoLONG{$iMIB} = $sName;
-      $LONGtoMIB{$sName} = $iMIB;
-      $SHORTtoMIB{&strip($sName)} = $iMIB;
+      $hsLongnameOfMIB{$iMIB} = $sName;
+      $hsMIBofLongname{$sName} = $iMIB;
+      $hsMIBofShortname{&_strip($sName)} = $iMIB;
       if ($mimename)
         {
         print STDERR " +   found mime: mib=$iMIB, mime=$sName\n" if $iDebug;
-	$MIBtoMIME{$iMIB} = $sName;
+	$hsMIMEofMIB{$iMIB} = $sName;
         # We've got the MIME name for this entry, don't look for more
         # until next entry:
         $mimename = 0;
@@ -924,11 +931,11 @@ while (1)
       if ($sAlias !~ m!None!i)
         {
         print STDERR " +   map Alias ($sAlias) to mib $iMIB\n" if $iDebug;
-        $SHORTtoMIB{&strip($sAlias)} = $iMIB;
+        $hsMIBofShortname{&_strip($sAlias)} = $iMIB;
 	if ($mimename)
 	  {
           print STDERR " +   found mime: mib=$iMIB, mime=$sAlias\n" if $iDebug;
-	  $MIBtoMIME{$iMIB} = $sAlias;
+	  $hsMIMEofMIB{$iMIB} = $sAlias;
           # We've got the MIME name for this entry, don't look for
           # more until next entry:
           $mimename = 0;
@@ -946,13 +953,14 @@ while (1)
  EQUAL_LINE:
   foreach my $sLine (@asEqualLines)
     {
+    next if ($sLine =~ m!\A#!);
     # print STDERR " +   equal-sign line $sLine...\n";
     my @as = split(/\ ===\ /, $sLine);
     next unless defined $as[0];
     next unless $as[0] ne '';
     next unless defined $as[1];
     next unless $as[1] ne '';
-    my $iMIB = $SHORTtoMIB{&strip($as[0])} || '';
+    my $iMIB = $hsMIBofShortname{&_strip($as[0])} || '';
     unless ($iMIB ne '')
       {
       print STDERR " --- can not find IANA entry for equal-sign directive $as[0]\n";
@@ -960,9 +968,9 @@ while (1)
       } # unless
     foreach my $s (@as)
       {
-      my $sStrip = &strip($s);
+      my $sStrip = &_strip($s);
       # print STDERR " +     $sStrip --> $iMIB\n";
-      $SHORTtoMIB{$sStrip} = $iMIB;
+      $hsMIBofShortname{$sStrip} = $iMIB;
       } # foreach
     } # foreach
 
@@ -999,11 +1007,11 @@ while (1)
           # $iDebug = 1;
           $iMIB = $sFakeMIB++;
           print STDERR " +   had to use a dummy mib ($iMIB) for U::Map8==$sMap8==\n" if $iDebug;
-          $LONGtoMIB{$sMap8} = $iMIB;
+          $hsMIBofLongname{$sMap8} = $iMIB;
           } # unless
         else
           {
-          $iMIB = $LONGtoMIB{$sFound};
+          $iMIB = $hsMIBofLongname{$sFound};
           print STDERR " +   found IANA name $sFound ($iMIB) for Map8 entry $sMap8\n" if $iDebug;
           }
         # $iDebug = ($iMIB =~ m!225[23]!);
@@ -1013,9 +1021,9 @@ while (1)
         # Map ALL the Map8 aliases to this Map8 master name:
         foreach my $s ($sMap8, @as)
           {
-          $s = &strip($s);
-          print STDERR " +      map $s to $iMIB in SHORTtoMIB...\n" if $iDebug;
-          $SHORTtoMIB{$s} = $iMIB;
+          $s = &_strip($s);
+          print STDERR " +      map $s to $iMIB in hsMIBofShortname...\n" if $iDebug;
+          $hsMIBofShortname{$s} = $iMIB;
           } # foreach
         # $iDebug = 0;
         } # while
@@ -1041,16 +1049,16 @@ while (1)
     my $sMapFile = "$MAP_Path/REGISTRY";
     if (open MAPS, $sMapFile)
       {
-      local $/;
-      undef $/;
+      local $/ = undef;
       my @asMAPS = split(/\n\s*\n/, <MAPS>);
+ UMAP_ENTRY:
       foreach my $sEntry (@asMAPS)
         {
         $iDebug = 0;
         # print STDERR " + working on Umap entry >>>>>$sEntry<<<<<...\n";
         my ($sName, $iMIB) = ('', '');
         # Get the value of the name field, and skip entries with no name:
-        next unless $sEntry =~ m!^name:\s+(\S+)!mi;
+        next UMAP_ENTRY unless $sEntry =~ m!^name:\s+(\S+)!mi;
         $sName = $1;
         # $iDebug = ($sName =~ m!apple!);
         print STDERR " +   UMAP sName is $sName\n" if $iDebug;
@@ -1065,16 +1073,16 @@ while (1)
           {
           # This entry does not have the MIB listed.  See if the name
           # of any of the aliases are known to our iana tables:
- ALIAS:
+ UMAP_ALIAS:
           foreach my $sAlias ($sName, @asAlias)
             {
             print STDERR " +     try alias $sAlias\n" if $iDebug;
-            my $iMIBtry = &short_to_mib(&strip($sAlias));
+            my $iMIBtry = &_short_to_mib(&_strip($sAlias));
             if ($iMIBtry)
               {
               print STDERR " +       matched\n" if $iDebug;
               $iMIB = $iMIBtry;
-              last ALIAS;
+              last UMAP_ALIAS;
               } # if
             } # foreach
           # If nothing matched, create a dummy mib:
@@ -1088,16 +1096,14 @@ while (1)
         # $iDebug = ($iMIB eq '17');
         print STDERR " +   UMAP mib is $iMIB\n" if $iDebug;
         $MIBtoUMAP{$iMIB} = $sName;
-        # Set the long IANA name, IF it is empty so far:
-        $MIBtoLONG{$iMIB} ||= $sName;
-        $LONGtoMIB{$sName} ||= $iMIB;
-        $SHORTtoMIB{&strip($sName)} ||= $iMIB;
+        $hsMIBofLongname{$sName} ||= $iMIB;
+        $hsMIBofShortname{&_strip($sName)} ||= $iMIB;
         foreach my $sAlias (@asAlias)
           {
           print STDERR " +   UMAP alias $sAlias\n" if $iDebug;
-          $SHORTtoMIB{&strip($sAlias)} = $iMIB;
-          } # while
-        } # foreach
+          $hsMIBofShortname{&_strip($sAlias)} = $iMIB;
+          } # foreach $sAlias
+        } # foreach UMAP_ENTRY
       close MAPS;
       # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{$sDummy .'029'}\n\n";
       } # if open
@@ -1160,7 +1166,7 @@ UMU8_NAME:
 
   print STDERR " + the following IANA names do *not* have entries in the Map8 table:\n";
   my %hiTried = ();
-  foreach my $sIANA (sort values %MIBtoLONG)
+  foreach my $sIANA (sort values %hsLongnameOfMIB)
     {
     next if $hiTried{$sIANA};
     print "$sIANA\n" unless defined &map8_charset_name($sIANA);
@@ -1187,18 +1193,18 @@ UMU8_NAME:
   last INFINITE;
 
   # debugging: huge dump:
-  # &dumpHash('MIBtoLONG', \%MIBtoLONG);
-  # &dumpHash('LONGtoMIB', \%LONGtoMIB);
-  # &dumpHash('SHORTtoMIB', \%SHORTtoMIB);
-  foreach (keys %SHORTtoMIB)
+  # &_dump_hash('hsLongnameOfMIB', \%hsLongnameOfMIB);
+  # &_dump_hash('hsMIBofLongname', \%hsMIBofLongname);
+  # &_dump_hash('hsMIBofShortname', \%hsMIBofShortname);
+  foreach (keys %hsMIBofShortname)
     {
-    print STDERR " + short_to_long($_) == ", &short_to_long($_) || 'undef', "\n";
+    print STDERR " + _short_to_long($_) == ", &_short_to_long($_) || 'undef', "\n";
     } # foreach
 
   last INFINITE;
   } # while
 
-sub dumpHash
+sub _dump_hash
   {
   my ($sName, $rh) = @_;
   print STDERR " + after initialization, $sName is:\n";
@@ -1206,7 +1212,7 @@ sub dumpHash
     {
     print STDERR " +   $key => $$rh{$key}\n";
     } # foreach
-  } # dumpHash
+  } # _dump_hash
 
 sub _init_data_extra
   {
@@ -1233,15 +1239,14 @@ EBCDIC-ES === ebcdic-cp-es
 EBCDIC-FR === ebcdic-cp-fr
 EBCDIC-IT === ebcdic-cp-it
 EBCDIC-UK === ebcdic-cp-gb
-EBCDIC-US === ebcdic-cp-us
 EBCDIC-FI-SE === ebcdic-cp-fi
 UTF-7 === Unicode-2-0-utf-7
 UTF-8 === Unicode-2-0-utf-8
 Extended_UNIX_Code_Packed_Format_for_Japanese === euc === euc-jp
-# These were added for Unicode::MapUTF8:
+# These are for Unicode::MapUTF8:
 ISO-10646-UCS-2 === ucs2
 ISO-10646-UCS-4 === ucs4
-# These were added for iconv:
+# These are for iconv:
 ISO-2022-JP === ISO-2022-JP-1
 # These are for Encode:
 IBM1047 === cp1047
@@ -1251,6 +1256,37 @@ JIS_X0201 === jis0201-raw
 JIS_C6226-1983 === jis0208-raw
 JIS_X0212-1990 === jis0212-raw
 KS_C_5601-1987 === ksc5601-raw
+CP037 === CP37
+cp863 === DOSCanadaF
+cp860 === DOSPortuguese
+cp869 === DOSGreek2
+koi8-r === cp878
+# These encodings are handled by Encode, but I don't know what they are:
+# ??? === AdobeZdingbats
+# ??? === MacArabic
+# ??? === MacCentralEurRoman
+# ??? === MacChineseSimp
+# ??? === MacChineseTrad
+# ??? === MacCroatian
+# ??? === MacCyrillic
+# ??? === MacDingbats
+# ??? === MacFarsi
+# ??? === MacGreek
+# ??? === MacHebrew
+# ??? === MacIcelandic
+# ??? === MacJapanese
+# ??? === MacKorean
+# ??? === MacRomanian
+# ??? === MacRumanian
+# ??? === MacSami
+# ??? === MacThai
+# ??? === MacTurkish
+# ??? === MacUkrainian
+# ??? === MacVietnamese
+# ??? === cp1006
+# ??? === dingbats
+# ??? === nextstep
+# ??? === posix-bc
 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
   } # _init_data_extra
 
@@ -1260,10 +1296,11 @@ sub _init_data
   # This big piece of data is the original document from
   # http://www.iana.org/assignments/character-sets
   return <<'EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE';
+
 ===================================================================
 CHARACTER SETS
 
-(last updated 2004-01-26)
+(last updated 07 December 2006)
 
 These are the official names for character sets that may be used in
 the Internet and may be referred to in Internet documentation.  These
@@ -1297,13 +1334,17 @@ intended for vendor specific coded character sets.
 	2000-2999	Vendor
 
 The aliases that start with "cs" have been added for use with the
-Printer MIB (see RFC 1759) and contain the standard numbers along with
-suggestive names in order to facilitate applications that want to
-display the names in user interfaces.  The "cs" stands for character
-set and is provided for applications that need a lower case first
-letter but want to use mixed case thereafter that cannot contain any
-special characters, such as underbar ("_") and dash ("-").  If the
-character set is from an ISO standard, its cs alias is the ISO
+IANA-CHARSET-MIB as originally defined in RFC3808, and as currently
+maintained by IANA at http://www.iana.org/assignments/ianacharset-mib.
+Note that the ianacharset-mib needs to be kept in sync with this
+registry.  These aliases that start with "cs" contain the standard 
+numbers along with suggestive names in order to facilitate applications 
+that want to display the names in user interfaces.  The "cs" stands 
+for character set and is provided for applications that need a lower 
+case first letter but want to use mixed case thereafter that cannot 
+contain any special characters, such as underbar ("_") and dash ("-").  
+
+If the character set is from an ISO standard, its cs alias is the ISO
 standard number or name.  If the character set is not from an ISO
 standard, but is registered with ISO (IPSJ/ITSCJ is the current ISO
 Registration Authority), the ISO Registry number is specified as
@@ -1769,10 +1810,12 @@ Alias: latin4
 Alias: l4
 Alias: csISOLatin4
 
-Name: ECMA-cyrillic                                      [RFC1345,KXS2]
+Name: ECMA-cyrillic                                     
 MIBenum: 77
-Source: ECMA registry
+Source: ISO registry (formerly ECMA registry)
+         http://www.itscj.ipsj.jp/ISO-IR/111.pdf
 Alias: iso-ir-111
+Alias: KOI8-E
 Alias: csISO111ECMACyrillic
 
 Name: CSA_Z243.4-1985-1                                  [RFC1345,KXS2]
@@ -2982,11 +3025,30 @@ Alias: None
 
 Name: TIS-620
 MIBenum: 2259
-Source: Thai Industrial Standards Institute (TISI)	     [Tantsetthi]
+Source: Thai Industrial Standards Institute (TISI)                             [Tantsetthi]
 
 Name: HZ-GB-2312
 MIBenum: 2085
-Source: RFC 1842, RFC 1843                              [RFC1842, RFC1843]
+Source: RFC 1842, RFC 1843                                               [RFC1842, RFC1843]
+
+Name: ISO-11548-1
+MIBenum: 118 
+Source: See <http://www.iana.org/assignments/charset-reg/ISO-11548-1>            [Thibault]
+Alias: ISO_11548-1
+Alias: ISO_TR_11548-1
+Alias: csISO115481
+
+Name: KZ-1048
+MIBenum: 119 
+Source: See <http://www.iana.org/assignments/charset-reg/KZ-1048>      [Veremeev, Kikkarin]
+Alias: STRK1048-2002
+Alias: RK1048
+Alias: csKZ1048
+
+Name: BRF
+MIBenum: 2106
+Source: See <http://www.iana.org/assignments/charset-reg/BRF>                    [Thibault]
+Alias: csBRF
 
 
 REFERENCES
@@ -3080,6 +3142,8 @@ PEOPLE
 
 [Davis] Mark Davis, <mark@unicode.org>, April 2002.
 
+[Kikkarin] Sairan M. Kikkarin, <sairan@sci.kz>, 7 December 2006.
+
 [Lazhintseva] Katya Lazhintseva, <katyal@MICROSOFT.com>, May 1996.
 
 [Mahdi] Tamer Mahdi, <tamer@ca.ibm.com>, August 2000.
@@ -3105,10 +3169,14 @@ PEOPLE
 
 [Tantsetthi] Trin Tantsetthi, <trin@mozart.inet.co.th>, September 1998.
 
+[Thibault] Samuel Thibault, <samuel.thibault@ens-lyon.org>, 7 December 2006.
+
 [Tumasonis] Vladas Tumasonis, <vladas.tumasonis@maf.vu.lt>, August 2000.
 
 [Uskov] Alexander Uskov, <auskov@idc.kz>, September 2002.
 
+[Veremeev] Alexei Veremeev, <Alexey.Veremeev@oracle.com>, 7 December 2006.
+
 [Wendt] Chris Wendt, <christw@microsoft.com>, December 1999.
 
 [Yick] Nicky Yick, <cliac@itsd.gcn.gov.hk>, October 2000.
@@ -3127,3 +3195,4 @@ EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
 1;
 
 __END__
+
